unit phTreeInGridBA;

interface
uses
  SysUtils,Classes,phTree_BA,phGrid_BA,
  BoldDerivedHandle, BoldSubscription, BoldHandles,
  BoldCursorHandle,BoldElements, BoldSystem,
  phTree, phBabGridView,phBabGridContent, ExtCtrls,phDataEntityPack,BoldHandle,BoldAttributes,
  BoldVariableHandle,BoldSystemHandle,BoldControlPack   ;
type
  TTreeInGridBA=class(TComponent)
  private
    fColumnForTree: integer;
    fGrid_BA: TphGrid_BA;
    fTree_BA: TphTree_BA;
    fBoldSystemHandle: TBoldSystemHandle;
    fNodeList:TList;
    procedure SetGrid_BA(const Value: TphGrid_BA);
    procedure SetTree_BA(const Value: TphTree_BA);
    procedure FillListWithVisibleItems(aDataList: TphDataList;
      aBoldObjectList: TBoldObjectList;aNodeList:TList);
    procedure OnDeriveAndSubscribeGridContent(Sender: TComponent;
      RootValue: TBoldElement; ResultElement: TBoldIndirectElement;
      Subscriber: TBoldSubscriber);
    procedure SetBoldSystemHandle(const Value: TBoldSystemHandle);
    procedure GridOutOfDate;
    procedure TreeDeleteOrInsertItem(Caller: TObject;
      theDataEntity: TphDataEntity);
    procedure ButtonClick(theBabGridView: TBabGridView_Custom;
      theBabCell: TBabCell);
    function CellInset(theBabGridView: TBabGridView_Custom;
      theBabCell: TBabCell): Integer;
    function GridCellToTreeNode(theBabCell: TBabCell): TphDataEntity_Tree;
    function IsButtonExpanded(theBabGridView: TBabGridView_Custom;
      theBabCell: TBabCell): Boolean;
    function IsButtonTerminal(theBabGridView: TBabGridView_Custom;
      theBabCell: TBabCell): Boolean;
    function GetParentButtonCell(theBabGridView: TBabGridView_Custom;
      theBabCell: TBabCell): TBabCell;
    function TreeNodeToGridCell(
      theDataEntity: TphDataEntity_Tree;theReferenceBabCell:TBabCell): TBabCell;
    function ButtonHasNonHiddenChildren(theBabGridView: TBabGridView_Custom;
      theBabCell: TBabCell): Boolean;
  protected
    fBDHGridContent: TBoldDerivedHandle;
    fBCHGrid: TBoldCursorHandle;
    fBVHGridOutOfDate:TBoldVariableHandle;
    fListClass:TBoldObjectListClass;

    procedure InitConnections;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  public
    constructor Create(aOwner:TComponent);override;
    destructor  Destroy;override;

    property  ListClass:TBoldObjectListClass read fListClass write fListClass;
  published
    property  Grid_BA:TphGrid_BA read fGrid_BA write SetGrid_BA;
    property  Tree_BA:TphTree_BA read fTree_BA write SetTree_BA;

    property  ColumnForTree:integer read fColumnForTree write fColumnForTree;
    property  BoldSystemHandle:TBoldSystemHandle read fBoldSystemHandle write SetBoldSystemHandle;
  end;


implementation

{ TTreeInGridBA }

constructor TTreeInGridBA.Create(aOwner: TComponent);
begin
  inherited;
  fBDHGridContent:=TBoldDerivedHandle.Create(nil);
  fBCHGrid:=TBoldCursorHandle.Create(nil);
  fBVHGridOutOfDate:=TBoldVariableHandle.Create(nil);
  fNodeList:=TList.Create;
end;

destructor TTreeInGridBA.Destroy;
begin
  FreeAndNil(fBDHGridContent);
  FreeAndNil(fBCHGrid);
  FreeAndNil(fBVHGridOutOfDate);
  FreeAndNil(fNodeList);
  if Assigned(Tree_BA) then
  begin
    Tree_BA.OnInsertItem:=nil;
    Tree_BA.OnDeleteItem:=nil;
  end;

  inherited;
end;


procedure TTreeInGridBA.FillListWithVisibleItems(aDataList:TphDataList;
                                               aBoldObjectList:TBoldObjectList;
                                               aNodeList:TList);
var
  i:integer;
begin
  for i:=0 to aDataList.Count-1 do
  begin
    with (aDataList.Items[i] as TphDataEntity_Tree) do
    begin
      if TObject(UserReference) is TBoldFollower then
      begin
        aBoldObjectList.Add(TBoldFollower(UserReference).Element as TBoldObject);
        aNodeList.Add(aDataList.Items[i]);
        if Expanded then
          FillListWithVisibleItems(SubDataList,aBoldObjectList,aNodeList);
      end;
    end;
  end;

end;



procedure TTreeInGridBA.InitConnections;
begin
  if not (csDesigning in ComponentState) then
  begin
    fBDHGridContent.OnDeriveAndSubscribe:=OnDeriveAndSubscribeGridContent;
    fBCHGrid.RootHandle:=fBDHGridContent;
    fBVHGridOutOfDate.ValueTypeName := 'String';
    fBVHGridOutOfDate.StaticSystemHandle:=fBoldSystemHandle;

    if Assigned(Grid_BA) then
    begin
      Grid_BA.BoldHandle:=fBCHGrid;
      Grid_BA.Columns[ColumnForTree].TreeButton:=TRUE;
      Grid_BA.OnButtonClick:=ButtonClick;
      Grid_BA.OnCellInset:=CellInset;
      Grid_BA.OnIsButtonExpanded:=IsButtonExpanded;
      Grid_BA.OnIsButtonTerminal:=IsButtonTerminal;
      Grid_BA.OnGetParentButtonCell:=GetParentButtonCell;
      Grid_BA.OnButtonHasNonHiddenChildren:=ButtonHasNonHiddenChildren;
    end;

    if Assigned(Tree_BA) then
    begin
      Tree_BA.OnInsertItem:=TreeDeleteOrInsertItem;
      Tree_BA.OnDeleteItem:=TreeDeleteOrInsertItem;
    end;
    GridOutOfDate;
  end;
end;


function TTreeInGridBA.ButtonHasNonHiddenChildren(theBabGridView:TBabGridView_Custom;theBabCell:TBabCell):Boolean;
var
  dataEntity_Tree:TphDataEntity_Tree;
begin
  result:=TRUE;
  dataEntity_Tree:=GridCellToTreeNode(theBabCell);
  if Assigned(dataEntity_Tree) and dataEntity_Tree.Expanded then
  begin
    result:=dataEntity_Tree.Expanded and (dataEntity_Tree.SubDataList.Count>0);
  end;
end;

function TTreeInGridBA.IsButtonExpanded(theBabGridView:TBabGridView_Custom;theBabCell:TBabCell):Boolean;
var
  dataEntity_Tree:TphDataEntity_Tree;
begin
  result:=FALSE;
  dataEntity_Tree:=GridCellToTreeNode(theBabCell);
  if Assigned(dataEntity_Tree) then
    result:=dataEntity_Tree.Expanded;
  Assert(Assigned(dataEntity_Tree));
end;

function TTreeInGridBA.IsButtonTerminal(theBabGridView:TBabGridView_Custom;theBabCell:TBabCell):Boolean;
var
  dataEntity_Tree:TphDataEntity_Tree;
begin
  result:=FALSE;
  dataEntity_Tree:=GridCellToTreeNode(theBabCell);
  if Assigned(dataEntity_Tree) then
    result:=Tree_BA.IsTerminal(dataEntity_Tree);
end;



function TTreeInGridBA.GetParentButtonCell(theBabGridView:TBabGridView_Custom;theBabCell:TBabCell):TBabCell;
var
  dataEntity_Tree:TphDataEntity_Tree;
begin
  result:=nil;
  dataEntity_Tree:=GridCellToTreeNode(theBabCell);
  if Assigned(dataEntity_Tree) and
    Assigned(theBabCell) then
  begin
    if dataEntity_Tree.ParentNode<>nil then
      result:=TreeNodeToGridCell(dataEntity_Tree.ParentNode,theBabCell);
  end;
end;



function TTreeInGridBA.TreeNodeToGridCell(theDataEntity:TphDataEntity_Tree;
                                  theReferenceBabCell:TBabCell):TBabCell;
var
  index:Integer;
begin
  result:=nil;
  index:=fNodeList.IndexOf(theDataEntity);
  if index<>-1 then
  begin
    result:=theReferenceBabCell.getGridCell.CellMatris.getCell(
                      theReferenceBabCell.AxisContentItemX.ListIndex,
                      theReferenceBabCell.getGridCell.FixedRows+index);
  end;
end;

function TTreeInGridBA.GridCellToTreeNode(theBabCell:TBabCell):TphDataEntity_Tree;
var
  index:Integer;
begin
  result:=nil;
  index:=theBabCell.AxisContentItemY.ListIndex-Grid_BA.BabGridCellForViewer.FixedRows;
  if (theBabCell is TBabSimpleCell) and
    Grid_BA.CellHasButton(theBabCell as TBabSimpleCell) and
    (index>=0) and
    Assigned(Tree_BA) then
  begin
    result:=TphDataEntity_Tree(fNodeList[index]);
  end;
end;


function TTreeInGridBA.CellInset(theBabGridView:TBabGridView_Custom;theBabCell:TBabCell):Integer;
var
  dataEntity_Tree:TphDataEntity_Tree;
begin
  result:=0;
  dataEntity_Tree:=GridCellToTreeNode(theBabCell);
  if Assigned(dataEntity_Tree) then
    result:=dataEntity_Tree.Level;

end;

procedure TTreeInGridBA.ButtonClick(theBabGridView:TBabGridView_Custom;theBabCell:TBabCell);
var
  dataEntity_Tree:TphDataEntity_Tree;
begin
  dataEntity_Tree:=GridCellToTreeNode(theBabCell);
  if Assigned(dataEntity_Tree) then
  begin
    dataEntity_Tree.Expanded:=not dataEntity_Tree.Expanded;
    GridOutOfDate;
  end;
end;

procedure TTreeInGridBA.GridOutOfDate;
begin
  if fBVHGridOutOfDate.Value is TBAString then
  begin
    (fBVHGridOutOfDate.Value as TBAString).AsString:='XXX';
    (fBVHGridOutOfDate.Value as TBAString).AsString:='YYY';
  end;
  fBDHGridContent.MarkOutOfDate;

end;

procedure TTreeInGridBA.TreeDeleteOrInsertItem(Caller: TObject;
  theDataEntity: TphDataEntity);
begin
  GridOutOfDate;
end;



procedure TTreeInGridBA.OnDeriveAndSubscribeGridContent(Sender: TComponent;
      RootValue: TBoldElement; ResultElement: TBoldIndirectElement;
      Subscriber: TBoldSubscriber);
var
  boldObjectList:TBoldObjectList;
begin
  fNodeList.Clear;
  if Assigned(fListClass) then
  begin
    boldObjectList:=fListClass.Create;
    ResultElement.SetOwnedValue(boldObjectList);
    if Assigned(Tree_BA) then
      FillListWithVisibleItems(Tree_BA.RootDataEntities,boldObjectList,fNodeList);
  end;
  
  if Assigned(fBVHGridOutOfDate.Value) then
    fBVHGridOutOfDate.Value.AddSubscription(Subscriber,beValueChanged,beValueChanged)
  else
    fBVHGridOutOfDate.AddSubscription(Subscriber,beValueChanged,beValueChanged);

end;


procedure TTreeInGridBA.SetBoldSystemHandle(
  const Value: TBoldSystemHandle);
begin
  fBoldSystemHandle := Value;
  InitConnections;
  FreeNotification(Value);
end;

procedure TTreeInGridBA.SetGrid_BA(const Value: TphGrid_BA);
begin
  fGrid_BA := Value;
  InitConnections;
  FreeNotification(Value);
end;

procedure TTreeInGridBA.SetTree_BA(const Value: TphTree_BA);
begin
  fTree_BA := Value;
  InitConnections;
  FreeNotification(Value);
end;

procedure TTreeInGridBA.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited;
  if Operation=opRemove then
  begin
    if AComponent=Tree_BA then
      fTree_BA:=nil
    else if AComponent=Grid_BA then
      fGrid_BA:=nil
    else if AComponent=BoldSystemHandle then
      fBoldSystemHandle:=nil;
  end;
end;

end.
