unit phTree_BA;

interface
uses
  Classes,Controls,phTree,ComCtrls,SysUtils,imgList,Windows,phDataTraveser,
  BoldElements,
  BoldHandles{(*)}, BoldExpressionHandle,
  BoldElementHandleFollower,
  BoldControlPack,
  BoldNumericControlPack,
  BoldStringControlPack,
  BoldGenericListControlPack,
  BoldNodeControlPack,
  BoldControllerListControlPack,
  BoldSystemRT,
  phDataEntityPack,
  Graphics,
  BoldSystem,
  BoldControlPackDefs;



type


  TphTree_BA = class(TphTree)
  private
    function GetBoldHandle: TBoldElementHandle;
    procedure SetBoldHandle(const Value: TBoldElementHandle);
    procedure SetTreeController(const Value: TBoldTreeFollowerController);
    procedure _BeforeMakeUptoDate(aBoldFollower: TBoldFollower);
    function GetCurrentBoldElement: TBoldElement;
    function _OnGetContextType:TBoldElementTypeInfo;

  protected
    fHandleFollower: TBoldElementHandleFollower;
    fTreeController: TBoldTreeFollowerController;
    fCurrentlyExpandingNode : TphDataEntity_Tree;

    procedure Expand(Node: TTreeNode); override;
    procedure _OnBeforeCanEditItem(theObs:TphDataEntity);override;
    procedure _SelectionChanged(thephDataList:TphDataList;theDataEntity:TphDataEntity);override;


    procedure _DisplayText(Follower: TBoldFollower); virtual;
    procedure _DisplayIcon(Follower: TBoldFollower); virtual;
    procedure _InsertNode(Follower: TBoldFollower); virtual;
    procedure _DeleteNode(Follower: TBoldFollower); virtual;

    procedure _OnChangeDataEntity_Tree(theTree:TObject;theDataEntity:TphDataEntity);

    procedure DoStartDrag(var DragObject: TDragObject); override;
    procedure DoEndDrag(Target: TObject; X, Y: Integer); override;


  public
    constructor Create(theOwner:TComponent);override;
    destructor Destroy;override;

    function NeedsDummyNode(theDataEntity:TphDataEntity):Boolean;override;
    function IsTerminal(theDataEntity:TphDataEntity): Boolean;

    function DataEntityToElement(theDataEntity:TphDataEntity):TBoldElement;
    function DataEntityToFollower(theDataEntity:TphDataEntity):TBoldFollower;

    procedure GetSelectedElements(theList:TBoldList);
    
    property CurrentBoldElement: TBoldElement read GetCurrentBoldElement;
    property CurrentlyExpandingNode:TphDataEntity_Tree read fCurrentlyExpandingNode;
  published
    property BoldHandle: TBoldElementHandle read GetBoldHandle  write SetBoldHandle;
    property BoldProperties: TBoldTreeFollowerController read fTreeController write SetTreeController;

    property ReadOnly;
  end;



implementation

uses
  BoldGUI,phTree_BA_Common;

{ TphTree_BA }

procedure TphTree_BA._DeleteNode(Follower: TBoldFollower);
begin
  TphTree_BA_Common._DeleteNode(self,Follower,BoldNodeTextIndex);
end;

procedure TphTree_BA._DisplayIcon(Follower: TBoldFollower);
begin
  TphTree_BA_Common._DisplayIcon(Follower);
end;

procedure TphTree_BA._DisplayText(Follower: TBoldFollower);
begin
  TphTree_BA_Common._DisplayText(self,Follower,ReadOnly);
end;

procedure TphTree_BA._InsertNode(Follower: TBoldFollower);
begin
  TphTree_BA_Common._InsertNode(self,Follower,BoldNodeTextIndex);
end;


procedure TphTree_BA._OnBeforeCanEditItem(theObs: TphDataEntity);
begin
  TphTree_BA_Common._OnBeforeCanEditItem(theObs,ReadOnly);
end;

procedure TphTree_BA._OnChangeDataEntity_Tree(theTree: TObject;
  theDataEntity: TphDataEntity);
begin
  TphTree_BA_Common._OnChangeDataEntity_Tree(theTree,theDataEntity);
end;

constructor TphTree_BA.Create(theOwner: TComponent);
begin
  inherited Create(theOwner);
  fTreeController := TBoldTreeFollowerController.Create( Self );

  with fTreeController do
  begin
    OnAfterInsertItem := _InsertNode;
    OnBeforeDeleteItem := _DeleteNode;
    OnGetContextType := _OnGetContextType;
    OnIconChanged := _DisplayIcon;
    OnTextChanged := _DisplayText;
    BeforeMakeUptoDate := _BeforeMakeUptoDate;
  end;
  fHandleFollower := TBoldElementHandleFollower.Create(Owner, fTreeController);

  OnValueChanged:=_OnChangeDataEntity_Tree;

end;


function TphTree_BA._OnGetContextType:TBoldElementTypeInfo;
begin
  if assigned( BoldHandle ) then
  begin
    if assigned( BoldHandle.StaticBoldType ) then
      result := BoldHandle.StaticBoldType
    else
      result := BoldHandle.StaticSystemTypeInfo
  end
  else
    result := nil;
end;

procedure TphTree_BA._BeforeMakeUptoDate(aBoldFollower:TBoldFollower);
begin
  if Assigned(aBoldFollower.OwningFollower) then
    fCurrentlyExpandingNode:=(aBoldFollower.OwningFollower.ControlData as TphDataEntity_Tree);
end;

destructor TphTree_BA.Destroy;
begin
  FreeAndNil(fTreeController);
  FreeAndNil(fHandleFollower);
  inherited;
end;


procedure TphTree_BA.Expand(Node: TTreeNode);
begin
  fCurrentlyExpandingNode:=GetDataEntityFromNode(Node);
  TphTree_BA_Common.Expand(Self,GetDataEntityFromNode(Node),BoldNodeTextIndex);
  inherited Expand(Node);
end;

function TphTree_BA.GetBoldHandle: TBoldElementHandle;
begin
  Result := fHandleFollower.BoldHandle;
end;

procedure TphTree_BA.SetBoldHandle(const Value: TBoldElementHandle);
begin
  fHandleFollower.BoldHandle := value
end;

procedure TphTree_BA.SetTreeController(
  const Value: TBoldTreeFollowerController);
begin
  fTreeController.Assign(Value);
end;

procedure TphTree_BA._SelectionChanged(thephDataList: TphDataList;
  theDataEntity: TphDataEntity);
begin
  inherited _SelectionChanged(thephDataList,theDataEntity);
  TBoldFollower(theDataEntity.UserReference).Selected:=theDataEntity.Selected;
end;

function TphTree_BA.NeedsDummyNode(theDataEntity: TphDataEntity): Boolean;
begin
  result:=(TBoldFollower(theDataEntity.UserReference).SubFollowers[BoldNodeListIndex].Controller as TBoldGenericListController).Parts.Count>0;
end;

function TphTree_BA.IsTerminal(theDataEntity:TphDataEntity): Boolean;
begin
  Result := not NeedsDummyNode(theDataEntity);
end;

procedure TphTree_BA.DoEndDrag(Target: TObject; X, Y: Integer);
var
  aTrv:TDataListTrav;
begin
  inherited;
  aTrv:=TDataListTrav.CreateMulti(ListOfDataListListInstances);
  try
    aTrv.First;
    while not aTrv.EOF do
    begin
      if aTrv.DataEntity.Selected and Assigned(aTrv.DataEntity.UserReference) then
      begin
        TBoldFollower(aTrv.DataEntity.UserReference).Controller.EndDrag;
      end;
      aTrv.Next;
    end;
  finally
    aTrv.Free;
  end;
end;

procedure TphTree_BA.DoStartDrag(var DragObject: TDragObject);
var
  aTrv:TDataListTrav;
  aList: TBoldObjectList;
  aElement : TBoldElement;
begin
  inherited;
  aTrv:=TDataListTrav.CreateMulti(ListOfDataListListInstances);
  aList := TBoldObjectList.Create;
  try
    aTrv.First;
    while not aTrv.EOF do
    begin
      if aTrv.DataEntity.Selected and Assigned(aTrv.DataEntity.UserReference) then
      begin
        aElement := DataEntityToElement(aTrv.DataEntity);
        if aElement is TBoldObject then  // Text nodes do not hold BoldObjects
          aList.Add(aElement as TBoldObject);
      end;
      aTrv.Next;
    end;
    BoldGUIHandler.DraggedObjects.AddList(aList);
  finally
    aTrv.Free;
    FreeAndNil(aList);
  end;
end;



function TphTree_BA.DataEntityToElement(
  theDataEntity: TphDataEntity): TBoldElement;
begin
  result := TphTree_BA_Common.DataEntityToElement(theDataEntity);
end;

function TphTree_BA.DataEntityToFollower(
  theDataEntity: TphDataEntity): TBoldFollower;
begin
  result := TphTree_BA_Common.DataEntityToFollower(theDataEntity);
end;

function TphTree_BA.GetCurrentBoldElement : TBoldElement;
begin
  result := DataEntityToElement(CurrentDataEntity);
end;



procedure TphTree_BA.GetSelectedElements(theList: TBoldList);
begin
  TphTree_BA_Common.GetSelectedElements(fListOfDataListListInstances,theList);
end;

end.
