unit phTree_BA_Common;
(**
Common stuff that both phTree_BA and GantTree_BA needs and uses
**)

interface
uses
  Classes,phTree,SysUtils,phDataTraveser,
  BoldControlPack,
  BoldNumericControlPack,
  BoldStringControlPack,
  BoldGenericListControlPack,
  BoldNodeControlPack,
  BoldControllerListControlPack,
  BoldSystemRT,
  phDataEntityPack,
  BoldSystem,
  BoldElements,
  BoldControlPackDefs;

type
  TphTree_BA_Common=class
  private
  public
    class procedure _DeleteNode(aTree:TphTree_ComBasic;Follower: TBoldFollower;aStopFI:integer);
    class procedure _DisplayIcon(Follower: TBoldFollower);
    class procedure _DisplayText(aTree:TphTree_ComBasic;Follower: TBoldFollower;aReadOnly:Boolean);
    class procedure _InsertNode(aTree:TphTree_ComBasic;Follower: TBoldFollower;aStopFI:integer);
    class procedure _OnBeforeCanEditItem(theObs: TphDataEntity;aReadOnly:Boolean);
    class procedure _OnChangeDataEntity_Tree(theTree: TObject;
      theDataEntity: TphDataEntity);

    class procedure FollowerExpansion(aFollower: TBoldFollower;aStopFI:integer);
    class procedure HideIfNeeded(aDataEntity_Tree: TphDataEntity_Tree;aStopFI:integer);
    class function DataEntityToFollower(
      theDataEntity: TphDataEntity): TBoldFollower;
    class function DataEntityToElement(
      theDataEntity: TphDataEntity): TBoldElement;
    class procedure Expand(aTree: TphTree_ComBasic;
      aDataEntity_Tree: TphDataEntity_Tree;aStopFI:integer);
    class procedure GetSelectedElements(the2levList: TphDataList2lev;
      theList: TBoldList);
  end;


implementation
uses
  Graphics;


class function TphTree_BA_Common.DataEntityToElement(theDataEntity: TphDataEntity): TBoldElement;
begin
  result := nil;
  if Assigned(theDataEntity) then  // May be nil!
  begin
    // It's an error if the UserReference isn't a follower
    result:=TBoldFollower(theDataEntity.UserReference).{(1)FIX: Value}Element;
  end;
end;

class procedure TphTree_BA_Common.GetSelectedElements(the2levList:TphDataList2lev;theList: TBoldList);
var
  i,ii:integer;
  element:TBoldElement;
begin
  for i:=0 to the2levList.Count-1 do
  begin
    for ii:=0 to the2levList.Items[i].Count-1 do
    begin
      if the2levList.Items[i].Items[ii].Selected then
      begin
        element:=DataEntityToElement(the2levList.Items[i].Items[ii]);
        // You cant add elements to a BoldObjectList
        if ((theList is TBoldObjectList) and (element is TBoldObject)) or
          (theList is TBoldMemberList) then
          theList.Add(element);
      end;
    end;
  end;
end;

class procedure TphTree_BA_Common.Expand(aTree:TphTree_ComBasic;aDataEntity_Tree: TphDataEntity_Tree;aStopFI:integer);
var
  aFollower:TBoldFollower;
  aResult:Boolean;
begin
  if Assigned(aDataEntity_Tree) then
  begin
    aFollower:=TBoldFollower(aDataEntity_Tree.UserReference);
    aResult := Assigned(aFollower) and
              ((aTree.MaxLevels=-1) or (aDataEntity_Tree.Level<aTree.MaxLevels));
    if aResult then
    begin
      FollowerExpansion(aFollower,aStopFI);
    end;
  end;
end;

class procedure TphTree_BA_Common._DeleteNode(aTree:TphTree_ComBasic;Follower: TBoldFollower;aStopFI:integer);
var
  p:TphDataEntity_Tree;
begin
  if Follower.ControlData is TphDataEntity_Tree then
  begin
    p:=(Follower.ControlData as TphDataEntity_Tree).ParentNode;
    aTree.RemoveDataEntity(Follower.ControlData as TphDataEntity_Tree);
    if Assigned(p) then
      TphTree_BA_Common.HideIfNeeded(p,aStopFI);
  end;
end;

class procedure TphTree_BA_Common._DisplayIcon(Follower: TBoldFollower);
begin
  if Follower.OwningFollower.ControlData is TphDataEntity_Tree then
  begin
    (Follower.OwningFollower.ControlData as TphDataEntity_Tree).ImageIndex:=(Follower.Controller as TBoldIntegerFollowerController).GetCurrentAsInteger( Follower );
    (Follower.OwningFollower.ControlData as TphDataEntity_Tree).SelectedIndex:=(Follower.Controller as TBoldIntegerFollowerController).GetCurrentAsInteger( Follower );
  end;
end;

class procedure TphTree_BA_Common._DisplayText(aTree:TphTree_ComBasic;Follower: TBoldFollower;aReadOnly:Boolean);
var
  aFont:TFont;
  aColor:TColor;
begin
  if Follower.OwningFollower.ControlData is TphDataEntity_Tree then
  begin
    aFont:=TFont.Create;
    try
      with (Follower.OwningFollower.ControlData as TphDataEntity_Tree) do
      begin
        Text:=(Follower.Controller as TBoldStringFollowerController).GetCurrentAsString( Follower );
        CanEdit:=(Follower.Controller as TBoldStringFollowerController).MayModify(Follower) and (not aReadOnly);

        (Follower.Controller as TBoldStringFollowerController).SetColor(aColor,clBlack,Follower);
        (Follower.Controller as TBoldStringFollowerController).SetFont(aFont,aTree.Font,Follower);

        FontStyle:=aFont.Style;
        FontName:=aFont.Name;
        //Color:=;
      end;
    finally
      aFont.Free;
    end;
  end;
end;

class procedure TphTree_BA_Common._InsertNode(aTree:TphTree_ComBasic;Follower: TBoldFollower;aStopFI:integer);
var
  aParentNode:TphDataEntity_Tree;
begin
  aParentNode:=nil;
  if Assigned( Follower.owningFollower.owningFollower ) then
  begin
    aParentNode:=Follower.OwningFollower.OwningFollower.ControlData as TphDataEntity_Tree;
  end;

  Follower.ControlData:=aTree.InsertDataEntity(aParentNode,Follower.index);
  (Follower.ControlData as TphDataEntity_Tree).UserReference:=Follower;
  (Follower.Controller as TBoldNodeFollowerController).SetActiveRange( Follower, BoldNodeIconIndex, aStopFI );

  Follower.EnsureDisplayable;

  TphTree_BA_Common.HideIfNeeded(Follower.ControlData as TphDataEntity_Tree,aStopFI);

end;


class procedure TphTree_BA_Common._OnBeforeCanEditItem(theObs: TphDataEntity;aReadOnly:Boolean);
var
  aFollower:TBoldFollower;
begin
  aFollower:=TBoldFollower(theObs.UserReference);
  theObs.CanEdit:=(aFollower.Controller as TBoldNodeFollowerController).Items[BoldNodeTextIndex].MayModify(aFollower.SubFollowers[BoldNodeTextIndex]) and (not aReadOnly);
end;

class procedure TphTree_BA_Common._OnChangeDataEntity_Tree(theTree: TObject;
  theDataEntity: TphDataEntity);
var
  aFollower:TBoldFollower;
begin
  aFollower:=TBoldFollower(theDataEntity.UserReference);
  if Assigned(aFollower) then
    ((aFollower.Controller as TBoldNodeFollowerController).Items[BoldNodeTextIndex] as TBoldStringFollowerController).SetAsString((theDataEntity as TphDataEntity_Tree).Text,aFollower);
end;





class function TphTree_BA_Common.DataEntityToFollower(
  theDataEntity: TphDataEntity): TBoldFollower;
begin
  result := nil;
  if Assigned(theDataEntity) then
    result:=TBoldFollower(theDataEntity.UserReference);
end;

class procedure TphTree_BA_Common.FollowerExpansion(aFollower:TBoldFollower;aStopFI:integer);
begin
  if ((aFollower.SubFollowerCount=0) or (not aFollower.SubFollowers[BoldNodeListIndex].Active)) then
  begin
    if (aFollower.OwningFollower.Controller as TBoldGenericListController).CanHaveSubFollowers then
    begin
      (aFollower.Controller as TBoldNodeFollowerController).SetActiveRange( aFollower, BoldNodeListIndex, aStopFI);
      aFollower.EnsureDisplayable;
    end;
  end;
end;

class procedure TphTree_BA_Common.HideIfNeeded(aDataEntity_Tree:TphDataEntity_Tree;aStopFI:integer);
var
  fl:TBoldFollower;
  exp:Boolean;
begin
  fl:=DataEntityToFollower(aDataEntity_Tree);
  if (fl.Controller as TBoldNodeFollowerController).HideNodeWithNoChildren then
  begin
    exp:=aDataEntity_Tree.Expanded;
    TphTree_BA_Common.FollowerExpansion(fl,aStopFI); // Fetch Immediatly to check if there are children
    if aDataEntity_Tree.SubDataList.Count=0 then
    begin
      aDataEntity_Tree.HideNode:=TRUE;
    end
    else
    begin
      aDataEntity_Tree.HideNode:=FALSE;
    end;
    aDataEntity_Tree.Expanded:=exp;
  end;

  if aDataEntity_Tree.ParentNode<>nil then
    HideIfNeeded(aDataEntity_Tree.ParentNode,aStopFI);
end;


end.
